import _has from "lodash/has";
import _get from "lodash/get";

var _templateObject;

function _taggedTemplateLiteral(strings, raw) { if (!raw) { raw = strings.slice(0); } return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

// @todo: remove the following line when part imports has been removed from this file
///<reference types="@sanity/types/parts" />
import React, { useCallback, useEffect, useImperativeHandle, useRef } from 'react';
import { FormField, FormFieldSet } from '@sanity/base/components';
import { Card, Select, TextInput } from '@sanity/ui';
import * as PathUtils from '@sanity/util/paths';
import { ChangeIndicatorProvider } from '@sanity/base/change-indicators';
import { PatchEvent, set, unset, setIfMissing } from 'part:@sanity/form-builder/patch-event';
import AceEditor from 'react-ace';
import styled from 'styled-components';
import { useId } from '@reach/auto-id';
import createHighlightMarkers, { highlightMarkersCSS } from './createHighlightMarkers';
import { LANGUAGE_ALIASES, ACE_EDITOR_PROPS, ACE_SET_OPTIONS, SUPPORTED_LANGUAGES, SUPPORTED_THEMES, DEFAULT_THEME, PATH_LANGUAGE, PATH_CODE, PATH_FILENAME } from './config';
/* eslint-disable import/no-unassigned-import */
// NOTE: MAKE SURE THESE ALIGN WITH SUPPORTED_LANGUAGES

import 'brace/mode/batchfile';
import 'brace/mode/css';
import 'brace/mode/html';
import 'brace/mode/typescript';
import 'brace/mode/javascript';
import 'brace/mode/json';
import 'brace/mode/jsx';
import 'brace/mode/markdown';
import 'brace/mode/php';
import 'brace/mode/sass';
import 'brace/mode/scss';
import 'brace/mode/python';
import 'brace/mode/sh';
import 'brace/mode/text';
import './groq';
import 'brace/theme/github';
import 'brace/theme/monokai';
import 'brace/theme/terminal';
import 'brace/theme/tomorrow';
/* eslint-enable import/no-unassigned-import */

var EditorContainer = styled(Card)(_templateObject || (_templateObject = _taggedTemplateLiteral(["\n  position: relative;\n  box-sizing: border-box;\n  overflow: hidden;\n  z-index: 0;\n\n  .ace_editor {\n    font-family: ", ";\n    font-size: ", ";\n    line-height: inherit;\n  }\n\n  ", "\n\n  &:not([disabled]):not([readonly]) {\n    &:focus,\n    &:focus-within {\n      box-shadow: 0 0 0 2px ", ";\n      background-color: ", ";\n      border-color: ", ";\n    }\n  }\n"])), _ref => {
  var theme = _ref.theme;
  return theme.sanity.fonts.code.family;
}, _ref2 => {
  var theme = _ref2.theme;
  return theme.sanity.fonts.code.sizes[1];
}, highlightMarkersCSS, _ref3 => {
  var theme = _ref3.theme;
  return theme.sanity.color.base.focusRing;
}, _ref4 => {
  var theme = _ref4.theme;
  return theme.sanity.color.base.bg;
}, _ref5 => {
  var theme = _ref5.theme;
  return theme.sanity.color.base.focusRing;
});

// Returns a string with the mode name if supported (because aliases), otherwise false
function isSupportedLanguage(mode) {
  var alias = LANGUAGE_ALIASES[mode];

  if (alias) {
    return alias;
  }

  var isSupported = SUPPORTED_LANGUAGES.find(lang => lang.value === mode);

  if (isSupported) {
    return mode;
  }

  return false;
}

var CodeInput = /*#__PURE__*/React.forwardRef((props, ref) => {
  var _props$value;

  var aceEditorRef = useRef();
  var aceEditorId = useId();
  var onFocus = props.onFocus,
      onChange = props.onChange,
      onBlur = props.onBlur,
      compareValue = props.compareValue,
      value = props.value,
      presence = props.presence,
      type = props.type,
      level = props.level,
      readOnly = props.readOnly,
      focusPath = props.focusPath;
  useImperativeHandle(ref, () => ({
    focus: () => {
      var _aceEditorRef$current, _aceEditorRef$current2;

      aceEditorRef === null || aceEditorRef === void 0 ? void 0 : (_aceEditorRef$current = aceEditorRef.current) === null || _aceEditorRef$current === void 0 ? void 0 : (_aceEditorRef$current2 = _aceEditorRef$current.editor) === null || _aceEditorRef$current2 === void 0 ? void 0 : _aceEditorRef$current2.focus();
    }
  }));
  var handleLanguageFocus = useCallback(() => {
    onFocus(PATH_LANGUAGE);
  }, [onFocus]);
  var handleCodeFocus = useCallback(() => {
    onFocus(PATH_CODE);
  }, [onFocus]);
  var handleFilenameFocus = useCallback(() => {
    onFocus(PATH_FILENAME);
  }, [onFocus]);
  var handleFilenameChange = useCallback(event => {
    var val = event.target.value;
    var path = PATH_FILENAME;
    onChange(PatchEvent.from([setIfMissing({
      _type: type.name
    }), val ? set(val, path) : unset(path)]));
  }, [onChange, type.name]);
  var getTheme = useCallback(() => {
    var preferredTheme = _get(type, 'options.theme');

    return preferredTheme && SUPPORTED_THEMES.find(theme => theme === preferredTheme) ? preferredTheme : DEFAULT_THEME;
  }, [type]);
  var handleToggleSelectLine = useCallback(lineNumber => {
    var _aceEditorRef$current3, _aceEditorRef$current4;

    var editorSession = (_aceEditorRef$current3 = aceEditorRef.current) === null || _aceEditorRef$current3 === void 0 ? void 0 : (_aceEditorRef$current4 = _aceEditorRef$current3.editor) === null || _aceEditorRef$current4 === void 0 ? void 0 : _aceEditorRef$current4.getSession();
    var backgroundMarkers = editorSession === null || editorSession === void 0 ? void 0 : editorSession.getMarkers(true);
    var currentHighlightedLines = Object.keys(backgroundMarkers).filter(key => backgroundMarkers[key].type === 'highlight').map(key => backgroundMarkers[key].range.start.row);
    var currentIndex = currentHighlightedLines.indexOf(lineNumber);

    if (currentIndex > -1) {
      // toggle remove
      currentHighlightedLines.splice(currentIndex, 1);
    } else {
      // toggle add
      currentHighlightedLines.push(lineNumber);
      currentHighlightedLines.sort();
    }

    onChange(PatchEvent.from(set(currentHighlightedLines.map(line => // ace starts at line (row) 0, but we store it starting at line 1
    line + 1), ['highlightedLines'])));
  }, [aceEditorRef, onChange]);
  var handleGutterMouseDown = useCallback(event => {
    var target = event.domEvent.target;

    if (target.classList.contains('ace_gutter-cell')) {
      var row = event.getDocumentPosition().row;
      handleToggleSelectLine(row);
    }
  }, [handleToggleSelectLine]);
  useEffect(() => {
    var _aceEditorRef$current5;

    var editor = aceEditorRef === null || aceEditorRef === void 0 ? void 0 : (_aceEditorRef$current5 = aceEditorRef.current) === null || _aceEditorRef$current5 === void 0 ? void 0 : _aceEditorRef$current5.editor;
    return () => {
      var _editor$session;

      editor === null || editor === void 0 ? void 0 : (_editor$session = editor.session) === null || _editor$session === void 0 ? void 0 : _editor$session.removeListener('guttermousedown', handleGutterMouseDown);
    };
  }, [aceEditorRef, handleGutterMouseDown]);
  var handleEditorLoad = useCallback(editor => {
    editor === null || editor === void 0 ? void 0 : editor.on('guttermousedown', handleGutterMouseDown);
  }, [handleGutterMouseDown]);
  var getLanguageAlternatives = useCallback(() => {
    var languageAlternatives = _get(type, 'options.languageAlternatives');

    if (!languageAlternatives) {
      return SUPPORTED_LANGUAGES;
    }

    if (!Array.isArray(languageAlternatives)) {
      throw new Error("'options.languageAlternatives' should be an array, got ".concat(typeof languageAlternatives));
    }

    return languageAlternatives.reduce((acc, _ref6) => {
      var title = _ref6.title,
          val = _ref6.value;
      var alias = LANGUAGE_ALIASES[val];

      if (alias) {
        // eslint-disable-next-line no-console
        console.warn("'options.languageAlternatives' lists a language with value \"%s\", which is an alias of \"%s\" - please replace the value to read \"%s\"", val, alias, alias);
        return acc.concat({
          title,
          value: alias
        });
      }

      if (!SUPPORTED_LANGUAGES.find(lang => lang.value === val)) {
        // eslint-disable-next-line no-console
        console.warn("'options.languageAlternatives' lists a language which is not supported: \"%s\", syntax highlighting will be disabled.", val);
      }

      return acc.concat({
        title,
        value: val
      });
    }, []);
  }, [type]);
  var handleCodeChange = useCallback(code => {
    var path = PATH_CODE;

    var fixedLanguage = _get(type, 'options.language');

    onChange(PatchEvent.from([setIfMissing({
      _type: type.name,
      language: fixedLanguage
    }), code ? set(code, path) : unset(path)]));
  }, [onChange, type]);
  var handleLanguageChange = useCallback(event => {
    var val = event.currentTarget.value;
    var path = PATH_LANGUAGE;
    onChange(PatchEvent.from([setIfMissing({
      _type: type.name
    }), val ? set(val, path) : unset(path)]));
  }, [onChange, type.name]);
  var languages = getLanguageAlternatives().slice();
  var selectedLanguage = props !== null && props !== void 0 && (_props$value = props.value) !== null && _props$value !== void 0 && _props$value.language ? languages.find(item => {
    var _props$value2;

    return item.value === (props === null || props === void 0 ? void 0 : (_props$value2 = props.value) === null || _props$value2 === void 0 ? void 0 : _props$value2.language);
  }) : undefined;
  var languageField = type.fields.find(field => field.name === 'language');
  var filenameField = type.fields.find(field => field.name === 'filename');
  var languageCompareValue = PathUtils.get(compareValue, PATH_LANGUAGE);
  var codeCompareValue = PathUtils.get(compareValue, PATH_CODE);
  var filenameCompareValue = PathUtils.get(compareValue, PATH_FILENAME);
  var languagePresence = presence.filter(presenceItem => PathUtils.startsWith(PATH_LANGUAGE, presenceItem.path));
  var codePresence = presence.filter(presenceItem => PathUtils.startsWith(PATH_CODE, presenceItem.path));
  var filenamePresence = presence.filter(presenceItem => PathUtils.startsWith(PATH_FILENAME, presenceItem.path));
  var renderEditor = useCallback(() => {
    var fixedLanguage = _get(type, 'options.language');

    var mode = isSupportedLanguage(value && value.language || fixedLanguage) || 'text';
    return /*#__PURE__*/React.createElement(EditorContainer, {
      radius: 1,
      shadow: 1,
      readOnly: readOnly
    }, /*#__PURE__*/React.createElement(AceEditor, {
      ref: aceEditorRef,
      mode: mode,
      theme: getTheme(),
      width: "100%",
      onChange: handleCodeChange,
      name: "editor-".concat(aceEditorId),
      value: value && value.code || '',
      markers: value && value.highlightedLines ? createHighlightMarkers(value.highlightedLines) : undefined,
      onLoad: handleEditorLoad,
      readOnly: readOnly,
      tabSize: 2,
      wrapEnabled: true,
      setOptions: ACE_SET_OPTIONS,
      editorProps: ACE_EDITOR_PROPS,
      onFocus: handleCodeFocus,
      onBlur: onBlur
    }));
  }, [type, value, readOnly, getTheme, handleCodeChange, aceEditorId, handleEditorLoad, handleCodeFocus, onBlur]);

  if (_has(type, 'options.language')) {
    return /*#__PURE__*/React.createElement(ChangeIndicatorProvider, {
      path: PATH_CODE,
      focusPath: focusPath,
      value: value === null || value === void 0 ? void 0 : value.code,
      compareValue: codeCompareValue
    }, /*#__PURE__*/React.createElement(FormFieldSet, {
      title: type.title,
      description: type.description,
      level: level
    }, renderEditor()));
  }

  return /*#__PURE__*/React.createElement(FormFieldSet, {
    title: type.title,
    description: type.description,
    level: level,
    __unstable_changeIndicator: false
  }, /*#__PURE__*/React.createElement(ChangeIndicatorProvider, {
    path: PATH_LANGUAGE,
    focusPath: focusPath,
    value: selectedLanguage === null || selectedLanguage === void 0 ? void 0 : selectedLanguage.value,
    compareValue: languageCompareValue
  }, /*#__PURE__*/React.createElement(FormField, {
    level: level + 1,
    label: (languageField === null || languageField === void 0 ? void 0 : languageField.title) || 'Language',
    __unstable_presence: languagePresence
  }, /*#__PURE__*/React.createElement(Select, {
    onChange: handleLanguageChange,
    readOnly: readOnly,
    value: (selectedLanguage === null || selectedLanguage === void 0 ? void 0 : selectedLanguage.value) || '',
    onFocus: handleLanguageFocus,
    onBlur: onBlur
  }, languages.map(lang => /*#__PURE__*/React.createElement("option", {
    key: lang.value,
    value: lang.value
  }, lang.title))))), _get(type, 'options.withFilename', false) && /*#__PURE__*/React.createElement(ChangeIndicatorProvider, {
    path: PATH_FILENAME,
    focusPath: focusPath,
    value: value === null || value === void 0 ? void 0 : value.filename,
    compareValue: filenameCompareValue
  }, /*#__PURE__*/React.createElement(FormField, {
    label: (filenameField === null || filenameField === void 0 ? void 0 : filenameField.title) || 'Filename',
    level: level + 1,
    __unstable_presence: filenamePresence
  }, /*#__PURE__*/React.createElement(TextInput, {
    name: "filename",
    value: (value === null || value === void 0 ? void 0 : value.filename) || '',
    placeholder: filenameField === null || filenameField === void 0 ? void 0 : filenameField.placeholder,
    onChange: handleFilenameChange,
    onFocus: handleFilenameFocus,
    onBlur: onBlur
  }))), /*#__PURE__*/React.createElement(ChangeIndicatorProvider, {
    path: PATH_CODE,
    focusPath: focusPath,
    value: value === null || value === void 0 ? void 0 : value.code,
    compareValue: codeCompareValue
  }, /*#__PURE__*/React.createElement(FormField, {
    label: "Code",
    level: level + 1,
    __unstable_presence: codePresence
  }, renderEditor())));
});
CodeInput.displayName = 'CodeInput';
export default CodeInput;