import _get from "lodash/get";

var _templateObject, _templateObject2;

function _taggedTemplateLiteral(strings, raw) { if (!raw) { raw = strings.slice(0); } return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

import React, { useCallback, useEffect, useRef } from 'react';
import AceEditor from 'react-ace';
import styled from 'styled-components';
import { Box } from '@sanity/ui';
import { SUPPORTED_LANGUAGES, LANGUAGE_ALIASES, ACE_EDITOR_PROPS, ACE_SET_OPTIONS } from './config';
import createHighlightMarkers from './createHighlightMarkers';

/* eslint-disable import/no-unassigned-import */
import 'brace/mode/batchfile';
import 'brace/mode/css';
import 'brace/mode/html';
import 'brace/mode/javascript';
import 'brace/mode/json';
import 'brace/mode/jsx';
import 'brace/mode/markdown';
import 'brace/mode/php';
import 'brace/mode/sass';
import 'brace/mode/scss';
import 'brace/mode/python';
import 'brace/mode/sh';
import 'brace/mode/text';
import 'brace/theme/github';
import 'brace/theme/monokai';
import 'brace/theme/terminal';
import 'brace/theme/tomorrow';
import './groq';
/* eslint-enable import/no-unassigned-import */

var PreviewContainer = styled(Box)(_templateObject || (_templateObject = _taggedTemplateLiteral(["\n  position: relative;\n"])));
var PreviewInner = styled(Box)(_templateObject2 || (_templateObject2 = _taggedTemplateLiteral(["\n  background-color: #272822;\n\n  .ace_editor {\n    box-sizing: border-box;\n    cursor: default;\n    pointer-events: none;\n  }\n\n  .ace_content {\n    box-sizing: border-box;\n    overflow: hidden;\n  }\n"])));

function isSupportedLanguage(mode) {
  var alias = LANGUAGE_ALIASES[mode];

  if (alias) {
    return alias;
  }

  var isSupported = SUPPORTED_LANGUAGES.find(lang => lang.value === mode);

  if (isSupported) {
    return mode;
  }

  return false;
}

export default function PreviewCode(props) {
  var aceEditorRef = useRef();
  useEffect(() => {
    var _aceEditorRef$current;

    if (!(aceEditorRef !== null && aceEditorRef !== void 0 && aceEditorRef.current)) return;
    var editor = (_aceEditorRef$current = aceEditorRef.current) === null || _aceEditorRef$current === void 0 ? void 0 : _aceEditorRef$current.editor;

    if (editor) {
      // Avoid cursor and focus tracking by Ace
      editor.renderer.$cursorLayer.element.style.opacity = 0;
      editor.textInput.getElement().disabled = true;
    }
  }, []);
  var handleEditorChange = useCallback(() => {// do nothing when the editor changes
  }, []);
  var value = props.value,
      type = props.type;

  var fixedLanguage = _get(type, 'options.language');

  var mode = isSupportedLanguage(value && value.language || fixedLanguage) || 'text';
  return /*#__PURE__*/React.createElement(PreviewContainer, null, /*#__PURE__*/React.createElement(PreviewInner, {
    padding: 4
  }, /*#__PURE__*/React.createElement(AceEditor, {
    ref: aceEditorRef,
    focus: false,
    mode: mode,
    theme: "monokai",
    width: "100%",
    onChange: handleEditorChange,
    maxLines: 200,
    readOnly: true,
    wrapEnabled: true,
    showPrintMargin: false,
    highlightActiveLine: false,
    cursorStart: -1,
    value: value && value.code || '',
    markers: value && value.highlightedLines ? createHighlightMarkers(value.highlightedLines) : undefined,
    tabSize: 2,
    showGutter: false,
    setOptions: ACE_SET_OPTIONS,
    editorProps: ACE_EDITOR_PROPS
  })));
}